package com.perforce.hive.connection;

import java.util.Enumeration;
import java.util.Properties;
import java.util.ResourceBundle;

public class ConnectionConfig {

	private static boolean init = false;
	private Properties properties;
	private static ConnectionConfig instance;

	/**
	 * Default constructor
	 */
	private ConnectionConfig(String name) {
		load(name);
	}

	public static ConnectionConfig get() {
		if (!init) {
			instance = new ConnectionConfig("config");
			init = true;
		}
		return instance;
	}

	public void load(String name) {
		ResourceBundle bundle = ResourceBundle.getBundle(name);

		properties = new Properties();
		Enumeration<String> keys = bundle.getKeys();
		while (keys.hasMoreElements()) {
			String key = keys.nextElement();
			properties.put(key, bundle.getString(key));
		}
	}
	
	public void set(String key, String value) {
		properties.put(key, value);
	}

	public String toUri() {
		String port = properties.getProperty("com.perforce.hive.p4port");
		if (port.startsWith("ssl:")) {
			String trim = port.substring(4, port.length());
			return "p4javassl://" + trim;
		}
		if (port.startsWith("rsh:")) {
			String trim = port.substring(4, port.length());
			return "p4jrsh://" + trim + " --java";
		}
		if (port.contains(":")) {
			return "p4java://" + port;
		}
		return "p4java://localhost:" + port;
	}

	public String getProtocol() {
		return properties.getProperty("com.perforce.hive.protocol");
	}
	
	public String getHost() {
		return properties.getProperty("com.perforce.hive.host");
	}
	
	public int getPort() {
		String port = properties.getProperty("com.perforce.hive.port");
		return Integer.parseInt(port);
	}

	public String getSearch() {
		return properties.getProperty("com.perforce.hive.p4search");
	}

	public String getClientPrefix() {
		return properties.getProperty("com.perforce.hive.clientprefix");
	}

	public String getProxyUser() {
		return properties.getProperty("com.perforce.hive.proxy.user");
	}

	public String getProxyTicket() {
		return properties.getProperty("com.perforce.hive.proxy.ticket");
	}

	public String getAnonymousUser() {
		return properties.getProperty("com.perforce.hive.anonymous.user");
	}

	public String getAnonymousTicket() {
		return properties.getProperty("com.perforce.hive.anonymous.ticket");
	}

	public String getProduct() {
		return properties.getProperty("com.perforce.hive.product");
	}

	public String getVersion() {
		return properties.getProperty("com.perforce.hive.version");
	}

	public int getProxyPool() {
		String poolStr = properties.getProperty("com.perforce.hive.proxy.poolsize");
		int pool = Integer.parseInt(poolStr);
		return pool;
	}

	public int getPoolTimeout() {
		String timeStr = properties.getProperty("com.perforce.hive.proxy.pooltimeout");
		int timeout = Integer.parseInt(timeStr);
		return timeout;
	}
	
	public String getKeystoreFile() {
		return properties.getProperty("com.perforce.hive.keystore.file");
	}
	
	public String getKeystorePass() {
		return properties.getProperty("com.perforce.hive.keystore.pass");
	}
	
	public String getTrustFile() {
		return properties.getProperty("com.perforce.hive.trust.file");
	}
	
	public String getTrustPass() {
		return properties.getProperty("com.perforce.hive.trust.pass");
	}

}
