package com.perforce.hive.depot;

import java.util.HashMap;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.Request;

public class DepotDomain {

	private static Logger logger = LoggerFactory.getLogger(DepotDomain.class);

	private static boolean init = false;
	private static DepotDomain instance = new DepotDomain();

	private HashMap<String, String> map;

	public static DepotDomain getInstance() {
		return instance;
	}

	private DepotDomain() {
		if (!init) {
			map = new HashMap<>();
			init = true;
		}
	}

	public void addDomain(String domain, String map) {
		this.map.put(domain, map);
	}

	public String getMap(Request request) {
		String domain = request.host();
		return map.get(domain);
	}

	/**
	 * Register redirect for depot domains.
	 *
	 * @param type
	 * @param base
	 *            - must start and end with a "/"
	 */
	public void registerDomain(ArtifactType type, String base) {
		String user = ConnectionConfig.get().getProxyUser();
		String ticket = ConnectionConfig.get().getProxyTicket();

		try {
			IOptionsServer p4 = ConnectionSession.getP4(user, ticket);
			List<DepotModel> depots = DepotFactory.listByType(p4, type);
			p4.disconnect();

			for (DepotModel depot : depots) {
				String host = ConnectionConfig.get().getHost();
				int port = ConnectionConfig.get().getPort();
				String name = depot.getName();
				String domain = name + "." + host + ":" + port;

				logger.info("Mapping domain: " + domain);
				map.put(domain, base + name);
			}
		} catch (P4JavaException e) {
			logger.error("Unable to register domains for ArtifactType: " + type);
			e.printStackTrace();
		}
	}
}
