package com.perforce.hive.site;

import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.api.repository.RepoInterface;
import com.perforce.hive.api.repository.SiteType;
import com.perforce.hive.artifact.proxy.ProxyArtifact;
import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.repository.RepoFormID;
import com.perforce.hive.store.DefaultStore;
import com.perforce.p4java.server.IOptionsServer;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClientBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import spark.Request;
import spark.Response;

import javax.servlet.http.HttpServletResponse;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Set;

public class ProxySite extends DefaultStore {

	private static Logger logger = LoggerFactory.getLogger(ProxySite.class);

	private final URL url;

	public ProxySite(Request request) throws MalformedURLException {
		super(request);

		String siteStr = RepoFormID.SITE.get(request);
		this.site = SiteType.parse(siteStr);
		this.url = new URL(RepoFormID.URL.get(request));
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("name: " + name + "\n");
		sb.append("url: " + url.toString() + "\n");
		return sb.toString();
	}

	@Override
	public String getSummaryPattern() {
		ArtifactType t = getArtifactType();
		type = t.getID();
		RepoInterface repo = PluginArtifactFactory.createStore(t);
		return repo.getSummaryPattern();
	}

	@Override
	public String getPath(Request request) {
		ArtifactType t = getArtifactType();
		type = t.getID();
		RepoInterface repo = PluginArtifactFactory.createStore(t);
		return repo.getPath(request);
	}

	@Override
	public HttpServletResponse fetch(IOptionsServer p4, Request request, Response response) throws Exception {

		// try to cache file if missing
		String path = getPath(request);
		String user = ConnectionConfig.get().getProxyUser();
		String ticket = ConnectionConfig.get().getProxyTicket();
		
		if (!isFile(p4, path)) {
			logger.info("[proxy] missing file: " + name + ":" + path);

			IOptionsServer px4 = ConnectionSession.getP4(user, ticket);
			ProxyArtifact artifact = new ProxyArtifact(px4, url, name, path);
			artifact.proxyUpload();
			artifact.index();
			artifact.close();
			logger.info("[proxy] indexed file: " + name + ":" + path);
		}

		return getFile(p4, path, response);
	}

	@Override
	public HttpResponse query(IOptionsServer p4, Request request, Response response) throws Exception {

		Set<String> map = request.queryParams();
		String param = "";
		for (String key : map) {
			param += (param.isEmpty()) ? "?" : "&";
			String value = request.queryParams(key);
			param += key + "=" + value;
		}

		String path = getPath(request);
		URI uri = new URI(url.toString() + "/" + path + param);
		logger.info("[proxy] query: " + name + ":" + uri);

		HttpClient client = HttpClientBuilder.create().build();
		HttpGet get = new HttpGet(uri);
		HttpResponse r = client.execute(get);

		return r;
	}

	public URL getUrl() {
		return url;
	}
}
