package com.perforce.hive.plugin.vagrant;

import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;

import com.perforce.hive.api.artifact.SummaryInterface;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.option.server.GetFileContentsOptions;
import com.perforce.p4java.server.IOptionsServer;

public class VagrantSummary implements SummaryInterface {

	private static SortedMap<String, String> mapParentID = new TreeMap<>();

	private SortedMap<String, String> properties = new TreeMap<>();

	static {
		mapParentID.put("/project/groupId", "GroupID");
		mapParentID.put("/project/artifactId", "ArtifactID");
		mapParentID.put("/project/version", "Version");
		mapParentID.put("/project/packaging", "Packaging");
		mapParentID.put("/project/name", "Name");
		mapParentID.put("/project/url", "Url");
		mapParentID.put("/project/description", "Description");
	}

	// TODO ######################## InputStream might be the Lowest Common Denominator
	
	
	public VagrantSummary(IOptionsServer p4, List<IFileSpec> summary) {
		try {
			GetFileContentsOptions printOpts = new GetFileContentsOptions();
			printOpts.setNoHeaderLine(true);
			InputStream ins = p4.getFileContents(summary, printOpts);

			// create a new document from input stream
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(ins);

			XPath xPath = XPathFactory.newInstance().newXPath();
			for (String id : mapParentID.keySet()) {
				String key = mapParentID.get(id);
				String value = xPath.compile(id).evaluate(doc);
				if(value!=null && !value.isEmpty()) {
					properties.put(key, value);
				}
			}
		} catch (Exception e) {
			// TODO
		}
	}

	@Override
	public Map<String, String> getProperties() {
		return properties;
	}
}
